%{

/*  Parse a custom state machine description language
 *  Copyright (C) 2000  Paul Stoffregen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <string.h>

#include "fsm.h"
#include "fsmparse.h"


char lex_sym_name[256];
int lineno=1;
struct symbol_struct *ptr;


int binary2int(char *str);

%}

%%

input_signals		{return INPUT_SIGNALS;}
output_signals		{return OUTPUT_SIGNALS;}
state_names		{return STATE_NAMES;}
default			{return DEFAULT;}
state			{return STATE;}
next			{return NEXT;}
if			{return IF;}
x			{
				yylval = -1;
				return NUM;
			}
[a-z][a-z0-9_]*		{
				/* keep name in lex_sym_name since yytext */
				/* could be overwritten if the parser does */
				/* a lookahead operation */
				strcpy(lex_sym_name, yytext);
				ptr = lookup_symbol(lex_sym_name);
				if (ptr == NULL) return SYM;
				if (ptr->type == TYPE_INPUT) return SYM_IN;
				if (ptr->type == TYPE_OUTPUT) return SYM_OUT;
				if (ptr->type == TYPE_STATE) return SYM_STATE;
				fprintf(stderr, "Error, symbol %s type is unknown!\n",
					lex_sym_name);
				return SYM;
			}

[01]+[bq]               {
                                yylval = binary2int(yytext);
                                return NUM;
                        }
0x[0-9a-f]+             {
                                sscanf(yytext, "%*c%*c%x", &yylval);
                                return NUM;
                        }
[0-9a-f]+[h]            {
                                sscanf(yytext, "%x%*[hH]", &yylval);
                                return NUM;
                        }
\$[0-9a-f]+             {
                                sscanf(yytext, "$%x", &yylval);
                                return NUM;
                        }
-?[0-9]+                {
                                sscanf(yytext, "%d", &yylval);
                                return NUM;
                        }

#[^\n]*			/* ignore comments */
[ \t\r]			/* ignore whitespace */
\n			{lineno++;}

.			{return *yytext;}

%%

/* turn a string like "10010110b" into an int */

int binary2int(char *str)
{
        register int i, j=1, sum=0;

        for (i=strlen(str)-2; i >= 0; i--) {
                sum += j * (str[i] == '1');
                j *= 2;
        }
        return (sum);
}



